#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "types.h"

/* defines section        */
#define PAGE_WRITE           0x02
#define STATUS_READ          0x05
#define WRITE_ENABLE         0x06
#define READ_FAST            0x0B
#define BULK_ERASE           0xC7

/* global variables */
extern HANDLE port_handle;

/* local variables */

/* None */

/* prototypes section */

/* None */

/*************************************************************************/
/*                                                                       */
/* Name: write_chars                                                     */
/*                                                                       */
/* Description: write bytes to serial port                               */
/*                                                                       */                 
/*                                                                       */
/*************************************************************************/
void write_chars(char *data, U32 size)
{
    DWORD write_len; 

    if (WriteFile(port_handle,              /* handle to file to write to         */
                  data,                     /* pointer to data to write to file   */
                  size,                     /* number of bytes to write           */
                  &write_len,NULL) == 0)    /* pointer to number of bytes written */
    {
        printf("Writing to serial communication has problem. \n");
    }
}

/*************************************************************************/
/*                                                                       */
/* Name: read_chars                                                      */
/*                                                                       */
/* Description: read bytes from serial port                              */
/*                                                                       */                 
/*                                                                       */
/*************************************************************************/
void read_chars(char *data, U32 size)
{
    DWORD read_len;  
    
    if (ReadFile(port_handle,  /* handle of file to read          */
                 data,         /* variable to store read data     */
                 size,         /* number of bytes to read         */
                 &read_len,    /* pointer to number of bytes read */
                 NULL) == 0)   /* pointer to structure for data   */
    {
        printf("Error reading from serial port !\n");
    }
}
/*************************************************************************/
/*                                                                       */
/* Name:         mc_read                                                 */
/*                                                                       */
/* Description:                                                          */
/*               read page in serial flash device                        */                  
/*                                                                       */
/*************************************************************************/
void mc_read(U32 offset, U8 *buffer, U32 len)
{
	char cmd[5]={READ_FAST, offset >> 16, offset >> 8, offset, 0};
    #if DEBUG
	    printf("read page %08x %02x %02x %02x \n", offset, cmd[1], cmd[2], cmd[3]);
	#endif 	
	write_chars(cmd, 5);
	read_chars(buffer, len);
}

/*************************************************************************/
/*                                                                       */
/* Name:         mc_program                                              */
/*                                                                       */
/* Description:                                                          */
/*               write page in serial flash device                       */                  
/*                                                                       */
/*************************************************************************/
void mc_program(U32 offset, U8 *buffer, S32 len)
{
	U8 cmd[5];
	U32 i; 
       
    /* send write enable and write page */
    cmd[0] = WRITE_ENABLE;
    cmd[1] = PAGE_WRITE;
    cmd[2] = offset >> 16;
    cmd[3] = offset >> 8;
    cmd[4] = offset;
    #if DEBUG
	    printf("write page %08x %02x %02x %02x \n", offset, cmd[2], cmd[3], cmd[4]);
	#endif
	write_chars(cmd, 5);
	Sleep(10);   

    /* max out is 64 byes on the ez-usb comm port (why ?)  */
    /* so write 4 * 64 byte packets = 256 bytes (one page) */
    for(i = 0; i < 4; i++)
    {
    	write_chars(buffer + (i * 64), 64);
    	Sleep(15);
	}
}

/*************************************************************************/
/*                                                                       */
/* Name:         mc_read_status                                          */
/*                                                                       */
/* Description:                                                          */
/*               read the status register of the serial flash device     */                  
/*                                                                       */
/*************************************************************************/
U8 mc_read_status(void)
{
	/* send the status command to the device */
	U8 cmd = STATUS_READ;
	write_chars(&cmd, 1);
		
	/* get its response */
	read_chars(&cmd, 1);
	
	return cmd;
}

/*************************************************************************/
/*                                                                       */
/* Name:         mc_erase_bulk                                           */
/*                                                                       */
/* Description:                                                          */
/*               erase the serial flash device to all 0xff               */                  
/*                                                                       */
/*************************************************************************/
void mc_erase_bulk(void)
{
    U8 cmd[2]; 
    
    /* send write enable and send block erase */
    cmd[0] = WRITE_ENABLE;
    cmd[1] = BULK_ERASE;
    write_chars(cmd, 2);     
}
